@@ -236,7 +236,7 @@ pub enum JsonStyle {
   Compact,
 }
 
-pub struct JSONPrinter<W: Write> {
+pub struct JSONPrinter<W: Write + Send + Sync> {
   output: Mutex<W>,
   style: JsonStyle,
   // indicate if any matches happened
@@ -248,7 +248,7 @@ impl JSONPrinter<Stdout> {
   }
 }
 
-impl<W: Write> JSONPrinter<W> {
+impl<W: Write + Send + Sync> JSONPrinter<W> {
   pub fn new(output: W, style: JsonStyle) -> Self {
     // no match happened yet
     Self {
@@ -300,7 +300,7 @@ impl<W: Write> JSONPrinter<W> {
   }
 }
 
-impl<W: Write> Printer for JSONPrinter<W> {
+impl<W: Write + Send + Sync> Printer for JSONPrinter<W> {
   fn print_rule<'a>(
     &self,
     matches: Matches!('a),
