@@ -1,5 +1,4 @@
 use dashmap::DashMap;
-use serde::{Deserialize, Serialize};
 use serde_json::Value;
 use tower_lsp::jsonrpc::Result;
 use tower_lsp::lsp_types::*;
@@ -10,7 +9,7 @@ use ast_grep_config::{CombinedScan, RuleCollection, RuleConfig};
 use ast_grep_core::{language::Language, AstGrep, Doc, Node, NodeMatch, StrDoc};
 
 use std::collections::HashMap;
-use std::path::PathBuf;
+use std::path::{Path, PathBuf};
 
 pub use tower_lsp::{LspService, Server};
 
@@ -25,48 +24,10 @@ struct VersionedAst<D: Doc> {
 pub struct Backend<L: LSPLang> {
   client: Client,
   map: DashMap<String, VersionedAst<StrDoc<L>>>,
+  base: PathBuf,
   rules: std::result::Result<RuleCollection<L>, String>,
 }
 
-#[derive(Serialize, Deserialize)]
-pub struct MatchRequest {
-  pattern: String,
-}
-
-#[derive(Serialize, Deserialize)]
-pub struct MatchResult {
-  uri: String,
-  position: Range,
-  content: String,
-}
-
-impl MatchResult {
-  fn new(uri: String, position: Range, content: String) -> Self {
-    Self {
-      uri,
-      position,
-      content,
-    }
-  }
-}
-
-impl<L: LSPLang> Backend<L> {
-  pub async fn search(&self, params: MatchRequest) -> Result<Vec<MatchResult>> {
-    let matcher = params.pattern;
-    let mut match_result = vec![];
-    for slot in self.map.iter() {
-      let uri = slot.key();
-      let versioned = slot.value();
-      for matched_node in versioned.root.root().find_all(matcher.as_str()) {
-        let content = matched_node.text().to_string();
-        let range = convert_node_to_range(&matched_node);
-        match_result.push(MatchResult::new(uri.clone(), range, content));
-      }
-    }
-    Ok(match_result)
-  }
-}
-
 const FALLBACK_CODE_ACTION_PROVIDER: Option<CodeActionProviderCapability> =
   Some(CodeActionProviderCapability::Simple(true));
 
@@ -222,12 +183,12 @@ fn convert_node_to_range<D: Doc>(node_match: &Node<D>) -> Range {
   }
 }
 
-fn get_non_empty_message<L: Language>(rule: &RuleConfig<L>) -> String {
+fn get_non_empty_message<L: Language>(rule: &RuleConfig<L>, nm: &NodeMatch<StrDoc<L>>) -> String {
   // Note: The LSP client in vscode won't show any diagnostics at all if it receives one with an empty message
   if rule.message.is_empty() {
     rule.id.to_string()
   } else {
-    rule.message.to_string()
+    rule.get_message(nm)
   }
 }
 fn convert_match_to_diagnostic<L: Language>(
@@ -246,7 +207,7 @@ fn convert_match_to_diagnostic<L: Language>(
       Severity::Hint => DiagnosticSeverity::HINT,
       Severity::Off => unreachable!("turned-off rule should not have match"),
     }),
-    message: get_non_empty_message(rule),
+    message: get_non_empty_message(rule, &node_match),
     source: Some(String::from("ast-grep")),
     tags: None,
     related_information: collect_labels(&node_match, uri),
@@ -295,31 +256,42 @@ fn build_error_id_to_ranges(diagnostics: Vec<Diagnostic>) -> HashMap<String, Vec
 }
 
 impl<L: LSPLang> Backend<L> {
-  pub fn new(client: Client, rules: std::result::Result<RuleCollection<L>, String>) -> Self {
+  pub fn new(
+    client: Client,
+    base: PathBuf,
+    rules: std::result::Result<RuleCollection<L>, String>,
+  ) -> Self {
     Self {
       client,
       rules,
+      base,
       map: DashMap::new(),
     }
   }
+
+  fn get_rules(&self, uri: &Url) -> Option<Vec<&RuleConfig<L>>> {
+    let absolute_path = uri.to_file_path().ok()?;
+    // for_path needs relative path, see https://github.com/ast-grep/ast-grep/issues/1272
+    let base = Path::new("./");
+    let path = if let Ok(p) = absolute_path.strip_prefix(&self.base) {
+      base.join(p)
+    } else {
+      absolute_path
+    };
+    let rules = self.rules.as_ref().ok()?.for_path(&path);
+    Some(rules)
+  }
+
   fn get_diagnostics(
     &self,
     uri: &Url,
     versioned: &VersionedAst<StrDoc<L>>,
   ) -> Option<Vec<Diagnostic>> {
-    let mut diagnostics = vec![];
-    let path = uri.to_file_path().ok()?;
-
-    let rules = match &self.rules {
-      Ok(rules) => rules.for_path(&path),
-      Err(_) => {
-        return Some(diagnostics);
-      }
-    };
-
+    let rules = self.get_rules(uri)?;
     let scan = CombinedScan::new(rules);
     let hit_set = scan.all_kinds();
     let matches = scan.scan(&versioned.root, hit_set, false).matches;
+    let mut diagnostics = vec![];
     for (id, ms) in matches {
       let rule = scan.get_rule(id);
       let to_diagnostic = |m| convert_match_to_diagnostic(m, rule, uri);
@@ -494,7 +466,10 @@ impl<L: LSPLang> Backend<L> {
     }
   }
 
-  fn on_apply_all_fix_impl(&self, first: Value) -> std::result::Result<WorkspaceEdit, LspError> {
+  async fn on_apply_all_fix_impl(
+    &self,
+    first: Value,
+  ) -> std::result::Result<WorkspaceEdit, LspError> {
     let text_doc: TextDocumentItem =
       serde_json::from_value(first).map_err(LspError::JSONDecodeError)?;
     let uri = text_doc.uri;
@@ -531,7 +506,7 @@ impl<L: LSPLang> Backend<L> {
       )
       .await;
     let first = arguments.first()?.clone();
-    let workspace_edit = match self.on_apply_all_fix_impl(first) {
+    let workspace_edit = match self.on_apply_all_fix_impl(first).await {
       Ok(workspace_edit) => workspace_edit,
       Err(error) => {
         self.report_error(error).await;
